@@ -90,8 +90,29 @@ struct _IdeHover
   guint dismiss_source;
 };
 
+static gboolean ide_hover_dismiss_cb (gpointer data);
+
 G_DEFINE_TYPE (IdeHover, ide_hover, G_TYPE_OBJECT)
 
+static void
+ide_hover_queue_dismiss (IdeHover *self)
+{
+  g_assert (IDE_IS_HOVER (self));
+
+  if (self->dismiss_source)
+    g_source_remove (self->dismiss_source);
+
+  /*
+   * Give ourselves just enough time to get the crossing event
+   * into the popover before we try to dismiss the popover.
+   */
+  self->dismiss_source =
+    gdk_threads_add_timeout_full (G_PRIORITY_HIGH,
+                                  1,
+                                  ide_hover_dismiss_cb,
+                                  self, NULL);
+}
+
 static void
 ide_hover_popover_closed_cb (IdeHover        *self,
                              IdeHoverPopover *popover)
@@ -132,6 +153,8 @@ ide_hover_popover_leave_notify_event_cb (IdeHover               *self,
                                          const GdkEventCrossing *event,
                                          IdeHoverPopover        *popover)
 {
+  GtkWidget *child;
+
   g_assert (IDE_IS_HOVER (self));
   g_assert (event != NULL);
   g_assert (IDE_IS_HOVER_POPOVER (popover));
@@ -139,6 +162,25 @@ ide_hover_popover_leave_notify_event_cb (IdeHover               *self,
   if (self->state == IDE_HOVER_STATE_IN_POPOVER)
     self->state = IDE_HOVER_STATE_DISPLAY;
 
+  /* If the window that we are crossing into is not a descendant of our
+   * popover window, then we want to dismiss. This is rather annoying to
+   * track and suffers the same issue as with GtkNotebook tabs containing
+   * buttons (where it's possible to break the prelight state tracking).
+   *
+   * In future Gtk releases, we may be able to use GtkEventControllerMotion.
+   */
+
+  if ((child = gtk_bin_get_child (GTK_BIN (popover))))
+    {
+      GdkRectangle point = { event->x, event->y, 1, 1 };
+      GtkAllocation alloc;
+
+      gtk_widget_get_allocation (child, &alloc);
+
+      if (!dzl_cairo_rectangle_contains_rectangle (&alloc, &point))
+        ide_hover_queue_dismiss (self);
+    }
+
   return GDK_EVENT_PROPAGATE;
 }
 
@@ -173,7 +215,8 @@ ide_hover_popover_destroy_cb (IdeHover        *self,
 static gboolean
 ide_hover_get_bounds (IdeHover    *self,
                       GtkTextIter *begin,
-                      GtkTextIter *end)
+                      GtkTextIter *end,
+                      GtkTextIter *hover)
 {
   GtkTextView *view;
   GtkTextIter iter;
@@ -182,13 +225,14 @@ ide_hover_get_bounds (IdeHover    *self,
   g_assert (IDE_IS_HOVER (self));
   g_assert (begin != NULL);
   g_assert (end != NULL);
+  g_assert (hover != NULL);
+
+  memset (begin, 0, sizeof *begin);
+  memset (end, 0, sizeof *end);
+  memset (hover, 0, sizeof *hover);
 
   if (!(view = dzl_signal_group_get_target (self->signals)))
-    {
-      memset (begin, 0, sizeof *begin);
-      memset (end, 0, sizeof *end);
-      return FALSE;
-    }
+    return FALSE;
 
   g_assert (GTK_IS_TEXT_VIEW (view));
 
@@ -201,6 +245,8 @@ ide_hover_get_bounds (IdeHover    *self,
   if (!gtk_text_view_get_iter_at_location (view, &iter, x, y))
     return FALSE;
 
+  *hover = iter;
+
   if (!_ide_source_iter_inside_word (&iter))
     {
       *begin = iter;
@@ -231,8 +277,10 @@ ide_hover_motion_timeout_cb (gpointer data)
   GdkRectangle rect;
   GdkRectangle begin_rect;
   GdkRectangle end_rect;
+  GdkRectangle hover_rect;
   GtkTextIter begin;
   GtkTextIter end;
+  GtkTextIter hover;
 
   g_assert (IDE_IS_HOVER (self));
 
@@ -246,7 +294,7 @@ ide_hover_motion_timeout_cb (gpointer data)
     return G_SOURCE_REMOVE;
 
   /* Make sure we're over text */
-  if (!ide_hover_get_bounds (self, &begin, &end))
+  if (!ide_hover_get_bounds (self, &begin, &end, &hover))
     return G_SOURCE_REMOVE;
 
   if (self->popover == NULL)
@@ -290,12 +338,15 @@ ide_hover_motion_timeout_cb (gpointer data)
   self->state = IDE_HOVER_STATE_DISPLAY;
   gtk_text_view_get_iter_location (GTK_TEXT_VIEW (view), &begin, &begin_rect);
   gtk_text_view_get_iter_location (GTK_TEXT_VIEW (view), &end, &end_rect);
+  gtk_text_view_get_iter_location (GTK_TEXT_VIEW (view), &hover, &hover_rect);
   gdk_rectangle_union (&begin_rect, &end_rect, &rect);
 
   gtk_text_view_buffer_to_window_coords (GTK_TEXT_VIEW (view),
                                          GTK_TEXT_WINDOW_WIDGET,
                                          rect.x, rect.y, &rect.x, &rect.y);
 
+  _ide_hover_popover_set_hovered_at (self->popover, &hover_rect);
+
   if (gtk_text_iter_equal (&begin, &end) &&
       gtk_text_iter_starts_line (&begin))
     {
@@ -458,18 +509,7 @@ ide_hover_leave_notify_event_cb (IdeHover               *self,
   if (should_ignore_event (view, event->window))
     return GDK_EVENT_PROPAGATE;
 
-  if (self->dismiss_source)
-    g_source_remove (self->dismiss_source);
-
-  /*
-   * Give ourselves just enough time to get the crossing event
-   * into the popover before we try to dismiss the popover.
-   */
-  self->dismiss_source =
-    gdk_threads_add_timeout_full (G_PRIORITY_HIGH,
-                                  1,
-                                  ide_hover_dismiss_cb,
-                                  self, NULL);
+  ide_hover_queue_dismiss (self);
 
   return GDK_EVENT_PROPAGATE;
 }
@@ -660,15 +700,42 @@ ide_hover_init (IdeHover *self)
                                    G_CONNECT_SWAPPED);
 }
 
-IdeHover *
-_ide_hover_new (IdeSourceView *view)
+static void
+ide_hover_extension_added_cb (IdeExtensionSetAdapter *set,
+                              PeasPluginInfo         *plugin_info,
+                              PeasExtension          *exten,
+                              gpointer                user_data)
 {
-  IdeHover *self;
+  IdeHover *self = user_data;
+  IdeHoverProvider *provider = (IdeHoverProvider *)exten;
+  IdeSourceView *view;
 
-  self = g_object_new (IDE_TYPE_HOVER, NULL);
-  dzl_signal_group_set_target (self->signals, view);
+  g_assert (IDE_IS_EXTENSION_SET_ADAPTER (set));
+  g_assert (IDE_IS_HOVER (self));
+  g_assert (plugin_info != NULL);
+  g_assert (IDE_IS_HOVER_PROVIDER (provider));
 
-  return self;
+  view = dzl_signal_group_get_target (self->signals);
+  ide_hover_provider_load (provider, view);
+}
+
+static void
+ide_hover_extension_removed_cb (IdeExtensionSetAdapter *set,
+                                PeasPluginInfo         *plugin_info,
+                                PeasExtension          *exten,
+                                gpointer                user_data)
+{
+  IdeHover *self = user_data;
+  IdeHoverProvider *provider = (IdeHoverProvider *)exten;
+  IdeSourceView *view;
+
+  g_assert (IDE_IS_EXTENSION_SET_ADAPTER (set));
+  g_assert (IDE_IS_HOVER (self));
+  g_assert (plugin_info != NULL);
+  g_assert (IDE_IS_HOVER_PROVIDER (provider));
+
+  view = dzl_signal_group_get_target (self->signals);
+  ide_hover_provider_unload (provider, view);
 }
 
 void
@@ -686,6 +753,20 @@ _ide_hover_set_context (IdeHover   *self,
                                                    IDE_TYPE_HOVER_PROVIDER,
                                                    "Hover-Provider-Languages",
                                                    NULL);
+
+  g_signal_connect (self->providers,
+                    "extension-added",
+                    G_CALLBACK (ide_hover_extension_added_cb),
+                    self);
+
+  g_signal_connect (self->providers,
+                    "extension-removed",
+                    G_CALLBACK (ide_hover_extension_removed_cb),
+                    self);
+
+  ide_extension_set_adapter_foreach (self->providers,
+                                     ide_hover_extension_added_cb,
+                                     self);
 }
 
 void
@@ -697,3 +778,14 @@ _ide_hover_set_language (IdeHover    *self,
   if (self->providers != NULL)
     ide_extension_set_adapter_set_value (self->providers, language);
 }
+
+IdeHover *
+_ide_hover_new (IdeSourceView *view)
+{
+  IdeHover *self;
+
+  self = g_object_new (IDE_TYPE_HOVER, NULL);
+  dzl_signal_group_set_target (self->signals, view);
+
+  return self;
+}
