@@ -52,6 +52,7 @@
 #include "runtimes/ide-runtime-manager.h"
 #include "search/ide-search-engine.h"
 #include "search/ide-search-provider.h"
+#include "session/ide-session.h"
 #include "snippets/ide-snippet-storage.h"
 #include "testing/ide-test-manager.h"
 #include "toolchain/ide-toolchain-manager.h"
@@ -122,6 +123,7 @@ struct _IdeContext
   IdeRuntimeManager        *runtime_manager;
   IdeToolchainManager      *toolchain_manager;
   IdeSearchEngine          *search_engine;
+  IdeSession               *session;
   IdeSnippetStorage        *snippets;
   IdeTestManager           *test_manager;
   IdeProject               *project;
@@ -134,6 +136,8 @@ struct _IdeContext
   IdeUnsavedFiles          *unsaved_files;
   IdeVcs                   *vcs;
 
+  GtkWidget                *workbench;
+
   IdeBuildLog              *log;
   guint                     log_id;
 
@@ -560,7 +564,32 @@ ide_context_dispose (GObject *object)
     }
 
   g_list_store_remove_all (self->pausables);
+
+  /* Request disposal of internal objects so that we help
+   * break any potential reference cycles.
+   */
+
   g_object_run_dispose (G_OBJECT (self->monitor));
+  g_object_run_dispose (G_OBJECT (self->debug_manager));
+  g_object_run_dispose (G_OBJECT (self->build_system));
+  g_object_run_dispose (G_OBJECT (self->device_manager));
+  g_object_run_dispose (G_OBJECT (self->run_manager));
+  g_object_run_dispose (G_OBJECT (self->run_manager));
+  g_object_run_dispose (G_OBJECT (self->runtime_manager));
+  g_object_run_dispose (G_OBJECT (self->toolchain_manager));
+  g_object_run_dispose (G_OBJECT (self->configuration_manager));
+  g_object_run_dispose (G_OBJECT (self->session));
+  g_object_run_dispose (G_OBJECT (self->unsaved_files));
+  g_object_run_dispose (G_OBJECT (self->vcs));
+  g_object_run_dispose (G_OBJECT (self->recent_manager));
+
+  if (self->workbench != NULL)
+    {
+      g_signal_handlers_disconnect_by_func (self->workbench,
+                                            G_CALLBACK (gtk_widget_destroyed),
+                                            &self->workbench);
+      self->workbench = NULL;
+    }
 
   G_OBJECT_CLASS (ide_context_parent_class)->dispose (object);
 
@@ -581,9 +610,9 @@ ide_context_finalize (GObject *object)
   g_clear_object (&self->downloads_dir);
   g_clear_object (&self->home_dir);
 
-  dzl_clear_pointer (&self->build_system_hint, g_free);
-  dzl_clear_pointer (&self->services_by_gtype, g_hash_table_unref);
-  dzl_clear_pointer (&self->recent_projects_path, g_free);
+  g_clear_pointer (&self->build_system_hint, g_free);
+  g_clear_pointer (&self->services_by_gtype, g_hash_table_unref);
+  g_clear_pointer (&self->recent_projects_path, g_free);
 
   g_clear_object (&self->build_system);
   g_clear_object (&self->configuration_manager);
@@ -593,7 +622,9 @@ ide_context_finalize (GObject *object)
   g_clear_object (&self->project);
   g_clear_object (&self->project_file);
   g_clear_object (&self->recent_manager);
+  g_clear_object (&self->run_manager);
   g_clear_object (&self->runtime_manager);
+  g_clear_object (&self->session);
   g_clear_object (&self->toolchain_manager);
   g_clear_object (&self->test_manager);
   g_clear_object (&self->unsaved_files);
@@ -909,6 +940,10 @@ ide_context_init (IdeContext *self)
 
   self->snippets = ide_snippet_storage_new ();
 
+  self->session = g_object_new (IDE_TYPE_SESSION,
+                                "context", self,
+                                NULL);
+
   IDE_EXIT;
 }
 
@@ -1231,10 +1266,11 @@ ide_context_init_unsaved_files (gpointer             source_object,
 
   task = ide_task_new (self, cancellable, callback, user_data);
   ide_task_set_source_tag (task, ide_context_init_unsaved_files);
+
   ide_unsaved_files_restore_async (self->unsaved_files,
                                    cancellable,
                                    ide_context_init_unsaved_files_cb,
-				   g_steal_pointer (&task));
+                                   g_steal_pointer (&task));
 }
 
 static void
@@ -1986,6 +2022,52 @@ ide_context_unload_configuration_manager (gpointer             source_object,
   IDE_EXIT;
 }
 
+static void
+ide_context_unload_session_save_cb (GObject      *object,
+                                    GAsyncResult *result,
+                                    gpointer      user_data)
+{
+  IdeSession *session = (IdeSession *)object;
+  g_autoptr(IdeTask) task = user_data;
+  g_autoptr(GError) error = NULL;
+
+  g_assert (IDE_IS_SESSION (session));
+  g_assert (G_IS_ASYNC_RESULT (result));
+  g_assert (IDE_IS_TASK (task));
+
+  /* unfortunate if this happens, but not much we can do */
+  if (!ide_session_save_finish (session, result, &error))
+    g_warning ("Failed to save session: %s", error->message);
+
+  ide_task_return_boolean (task, TRUE);
+}
+
+static void
+ide_context_unload_session (gpointer             source_object,
+                            GCancellable        *cancellable,
+                            GAsyncReadyCallback  callback,
+                            gpointer             user_data)
+{
+  IdeContext *self = source_object;
+  g_autoptr(IdeTask) task = NULL;
+
+  IDE_ENTRY;
+
+  g_assert (IDE_IS_CONTEXT (self));
+  g_assert (!cancellable || G_IS_CANCELLABLE (cancellable));
+  g_assert (IDE_IS_SESSION (self->session));
+
+  task = ide_task_new (self, cancellable, callback, user_data);
+  ide_task_set_source_tag (task, ide_context_unload_session);
+
+  ide_session_save_async (self->session,
+                          cancellable,
+                          ide_context_unload_session_save_cb,
+                          g_steal_pointer (&task));
+
+  IDE_EXIT;
+}
+
 static void
 ide_context_unload__unsaved_files_save_cb (GObject      *object,
                                            GAsyncResult *result,
@@ -2061,10 +2143,6 @@ ide_context_unload_cb (GObject      *object,
   g_assert (IDE_IS_TASK (unload_task));
   g_assert (IDE_IS_TASK (task));
 
-  g_clear_object (&self->device_manager);
-  g_clear_object (&self->runtime_manager);
-  g_clear_object (&self->toolchain_manager);
-
   if (!ide_task_propagate_boolean (unload_task, &error))
     ide_task_return_error (task, g_steal_pointer (&error));
   else
@@ -2086,6 +2164,7 @@ ide_context_do_unload_locked (IdeContext *self)
                         ide_task_get_cancellable (task),
                         ide_context_unload_cb,
                         g_object_ref (task),
+                        ide_context_unload_session,
                         ide_context_unload_configuration_manager,
                         ide_context_unload_buffer_manager,
                         ide_context_unload_unsaved_files,
@@ -2159,75 +2238,35 @@ ide_context_unload_finish (IdeContext    *self,
 
   ret = ide_task_propagate_boolean (IDE_TASK (result), error);
 
+  g_object_run_dispose (G_OBJECT (self));
+
   IDE_RETURN (ret);
 }
 
-static gboolean restore_in_idle (gpointer user_data);
-
 static void
-ide_context_restore__load_file_cb (GObject      *object,
-                                   GAsyncResult *result,
-                                   gpointer      user_data)
+ide_context_session_restore_cb (GObject      *object,
+                                GAsyncResult *result,
+                                gpointer      user_data)
 {
-  IdeBufferManager *buffer_manager = (IdeBufferManager *)object;
+  IdeSession *session = (IdeSession *)object;
   g_autoptr(IdeTask) task = user_data;
-  g_autoptr(IdeBuffer) buffer = NULL;
   g_autoptr(GError) error = NULL;
 
-  g_assert (IDE_IS_BUFFER_MANAGER (buffer_manager));
+  IDE_ENTRY;
+
+  g_assert (IDE_IS_SESSION (session));
+  g_assert (G_IS_ASYNC_RESULT (result));
   g_assert (IDE_IS_TASK (task));
 
-  if (!(buffer = ide_buffer_manager_load_file_finish (buffer_manager, result, &error)))
+  if (!ide_session_restore_finish (session, result, &error))
     {
-      g_warning ("%s", error->message);
-      /* TODO: add error into grouped error */
+      if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))
+        g_warning ("Failed to restore session: %s", error->message);
     }
 
-  g_idle_add (restore_in_idle, g_object_ref (task));
-}
+  ide_task_return_boolean (task, TRUE);
 
-static gboolean
-restore_in_idle (gpointer user_data)
-{
-  g_autoptr(IdeFile) ifile = NULL;
-  g_autoptr(IdeTask) task = user_data;
-  IdeUnsavedFile *uf;
-  IdeContext *self;
-  GPtrArray *ar;
-  GFile *file;
-
-  g_assert (IDE_IS_TASK (task));
-
-  self = ide_task_get_source_object (task);
-  g_assert (IDE_IS_CONTEXT (self));
-
-  ar = ide_task_get_task_data (task);
-
-  if (ar == NULL || ar->len == 0)
-    {
-      self->restoring = FALSE;
-      ide_task_return_boolean (task, TRUE);
-      return G_SOURCE_REMOVE;
-    }
-
-  g_assert (ar != NULL);
-  g_assert (ar->len > 0);
-
-  uf = g_ptr_array_index (ar, ar->len - 1);
-  file = ide_unsaved_file_get_file (uf);
-  ifile = ide_file_new (self, file);
-  g_ptr_array_remove_index (ar, ar->len - 1);
-
-  ide_buffer_manager_load_file_async (self->buffer_manager,
-                                      ifile,
-                                      FALSE,
-                                      IDE_WORKBENCH_OPEN_FLAGS_BACKGROUND,
-                                      NULL,
-                                      ide_task_get_cancellable (task),
-                                      ide_context_restore__load_file_cb,
-                                      g_object_ref (task));
-
-  return G_SOURCE_REMOVE;
+  IDE_EXIT;
 }
 
 void
@@ -2239,10 +2278,22 @@ ide_context_restore_async (IdeContext          *self,
   g_autoptr(IdeTask) task = NULL;
   g_autoptr(GPtrArray) ar = NULL;
 
+  IDE_ENTRY;
+
   g_return_if_fail (IDE_IS_CONTEXT (self));
   g_return_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable));
 
   task = ide_task_new (self, cancellable, callback, user_data);
+  ide_task_set_source_tag (task, ide_context_restore_async);
+
+  if (self->unloading)
+    {
+      ide_task_return_new_error (task,
+                                 G_IO_ERROR,
+                                 G_IO_ERROR_FAILED,
+                                 _("Context already unloading, cannot restore."));
+      IDE_EXIT;
+    }
 
   if (self->restored)
     {
@@ -2250,38 +2301,16 @@ ide_context_restore_async (IdeContext          *self,
                                  G_IO_ERROR,
                                  G_IO_ERROR_FAILED,
                                  _("Context has already been restored."));
-      return;
+      IDE_EXIT;
     }
 
   self->restored = TRUE;
-
-  ar = ide_unsaved_files_to_array (self->unsaved_files);
-  IDE_PTR_ARRAY_SET_FREE_FUNC (ar, ide_unsaved_file_unref);
-
-  if (ar->len == 0)
-    {
-      ide_task_return_boolean (task, TRUE);
-      return;
-    }
-
-  if (ar->len > RESTORE_FILES_MAX_FILES)
-    {
-      /*
-       * To protect from some insanity, ignore attempts to restore files if
-       * they are over RESTORE_FILES_MAX_FILES. Just prune and go back to
-       * normal.  This should help in situations where hadn't pruned the
-       * unsaved files list.
-       */
-      ide_unsaved_files_clear (self->unsaved_files);
-      ide_task_return_boolean (task, TRUE);
-      return;
-    }
-
   self->restoring = TRUE;
 
-  ide_task_set_task_data (task, g_ptr_array_ref (ar), (GDestroyNotify)g_ptr_array_unref);
-
-  g_idle_add (restore_in_idle, g_object_ref (task));
+  ide_session_restore_async (self->session,
+                             cancellable,
+                             ide_context_session_restore_cb,
+                             g_steal_pointer (&task));
 }
 
 gboolean
@@ -2289,12 +2318,16 @@ ide_context_restore_finish (IdeContext    *self,
                             GAsyncResult  *result,
                             GError       **error)
 {
-  IdeTask *task = (IdeTask *)result;
+  gboolean ret;
+
+  IDE_ENTRY;
 
   g_return_val_if_fail (IDE_IS_CONTEXT (self), FALSE);
-  g_return_val_if_fail (IDE_IS_TASK (task), FALSE);
+  g_return_val_if_fail (IDE_IS_TASK (result), FALSE);
 
-  return ide_task_propagate_boolean (task, error);
+  ret = ide_task_propagate_boolean (IDE_TASK (result), error);
+
+  IDE_RETURN (ret);
 }
 
 gboolean
@@ -2305,11 +2338,14 @@ _ide_context_is_restoring (IdeContext *self)
 
 /**
  * ide_context_get_settings:
+ * @self: an #IdeContext
+ * @schema_id: the #GSettings schema identifier
+ * @relative_path: (nullable): the path for the settings or %NULL
  *
  * Gets an #IdeSettings representing the given #GSettingsSchema.
  *
- * relative_path will be used to apply multiple layers of settings. Project settings will be
- * applied to first, followed by global settings.
+ * @relative_path will be used to apply multiple layers of settings.
+ * Project settings will be applied to first, followed by global settings.
  *
  * Returns: (transfer full): An #IdeSettings.
  */
@@ -2813,6 +2849,7 @@ ide_context_cache_filename (IdeContext  *self,
 IdeVcsMonitor *
 ide_context_get_monitor (IdeContext *self)
 {
+  g_return_val_if_fail (IDE_IS_MAIN_THREAD (), NULL);
   g_return_val_if_fail (IDE_IS_CONTEXT (self), NULL);
 
   if (self->monitor == NULL)
@@ -2928,3 +2965,38 @@ ide_context_is_unloading (IdeContext *self)
 
   return self->unloading;
 }
+
+/**
+ * ide_context_get_workbench:
+ * @self: a #IdeContext
+ *
+ * When a workbench has registered itself with this context, you can use this
+ * to get the #IdeWorkbench instance.
+ *
+ * Returns: (transfer none) (nullable): an #IdeWorkbench, or %NULL
+ *
+ * Since: 3.30
+ */
+GtkWidget *
+ide_context_get_workbench (IdeContext *self)
+{
+  g_return_val_if_fail (IDE_IS_CONTEXT (self), NULL);
+
+  return self->workbench;
+}
+
+void
+_ide_context_set_workbench (IdeContext *self,
+                            GtkWidget  *workbench)
+{
+  g_return_if_fail (IDE_IS_CONTEXT (self));
+  g_return_if_fail (GTK_IS_WIDGET (workbench));
+  g_return_if_fail (self->workbench == NULL);
+
+  self->workbench = workbench;
+
+  g_signal_connect (self->workbench,
+                    "destroy",
+                    G_CALLBACK (gtk_widget_destroyed),
+                    &self->workbench);
+}
