@@ -120,6 +120,8 @@ gbp_flatpak_runner_fixup_launcher (IdeRunner             *runner,
       ide_subprocess_launcher_insert_argv (launcher, i++, "--socket=wayland");
     }
 
+  ide_subprocess_launcher_insert_argv (launcher, i++, "--talk-name=org.freedesktop.portal.*");
+
   /* Proxy environment stuff to the launcher */
   if ((env = ide_runner_get_environment (runner)) &&
       (environ_ = ide_environment_get_environ (env)))
@@ -161,8 +163,8 @@ gbp_flatpak_runner_finalize (GObject *object)
 {
   GbpFlatpakRunner *self = (GbpFlatpakRunner *)object;
 
-  dzl_clear_pointer (&self->build_path, g_free);
-  dzl_clear_pointer (&self->binary_path, g_free);
+  g_clear_pointer (&self->build_path, g_free);
+  g_clear_pointer (&self->binary_path, g_free);
 
   G_OBJECT_CLASS (gbp_flatpak_runner_parent_class)->finalize (object);
 }
